//=============================================================================
// FA_EvidenceBook.js
//=============================================================================

/*:
 * @plugindesc Displays Key items as Evidence (Based on Yoji Ojima's ItemBook Plugin)
 * @author Harmless (Based on Yoji Ojima's ItemBook Plugin)
 *
 * @param Unknown Data
 * @desc The index name for an unknown item.
 * @default ??????
 *
 * @param Command Label
 * @desc The command tag used by scripting
 * @default EvidenceBook
 *
 * @param Var Id
 * @desc The ID of the variable in which the select evidence piece id will be noted (Buggy)
 * @default 5
 *
 * @param Debug Mode
 * @desc Enables console messages
 * @default false
 *
 * @help
 *
 * Plugin Command:
 *   EvidenceBook open inMenu 6    # Open the evidence book screen with inMenu view mode (Shows skillType 0) [id - 6 = nb]
 *   EvidenceBook open inGame 3    # Open the evidence book screen with inGame view mode (Shows objectType 3)
 *   EvidenceBook add 3            # Add item #3 to the evidence book
 *   EvidenceBook addskill 3       # Add skill #3 to the evidence book
 *   EvidenceBook remove 6         # Remove item #6 from the evidence book
 *   EvidenceBook complete         # Complete the evidence book
 *   EvidenceBook clear            # Clear the evidence book
 *
 */

(function() {

    var parameters = PluginManager.parameters('EvidenceBook');
    var unknownData = String(parameters['Unknown Data'] || '??????');
    var commandLabel = String(parameters['Command Label'] || 'EvidenceBook');
    var selectIndexVariable = Number(parameters['Var Id'] || 1);
    var debugMode = Boolean(parameters['Debug Mode'] || false);

    var allowSelect = false;
    
    var _Game_Interpreter_pluginCommand =
            Game_Interpreter.prototype.pluginCommand;
    Game_Interpreter.prototype.pluginCommand = function(command, args) {
        _Game_Interpreter_pluginCommand.call(this, command, args);
        if (command === commandLabel) {
            switch (args[0]) {
            case 'open':
                viewMode(String(args[1]));
                itemTypeIndex = Number(args[2] || 2);
                SceneManager.push(Scene_EvidenceBook);
                break;
            case 'add':
                $gameSystem.addToBook(Number(args[1]),false);
                break;
            case 'addskill':
                $gameSystem.addToBook(Number(args[1]),true);
                break;
            case 'remove':
                $gameSystem.removeFromBook(Number(args[1]),false);
                break;
            case 'removeskill':
                $gameSystem.removeFromBook(Number(args[1]),true);
                break;
            case 'complete':
                $gameSystem.completeBook();
                break;
            case 'clear':
                $gameSystem.clearBook();
                break;
            }
        }
    };
    
    viewMode = function(viewModeType){
        switch(viewModeType){
            case 'inMenu':
                allowSelect = false;
                Window_EvidenceBookIndex.lastTopRow = 0;
                Window_EvidenceBookIndex.lastIndex = 0;
                break;
            case 'inGame':
                allowSelect = true;
                Window_EvidenceBookIndex.lastTopRow = 0;
                Window_EvidenceBookIndex.lastIndex = 0;
                break;
                
            debugConsole(viewModeType);
        }
    }
    
    debugConsole = function(msg){
        if(debugMode){
            console.log(msg);
        }
    }

    // Game_System
    //////////////////////////////////////
    
    Game_System.prototype.addToBook = function(dataId, isSkill) {
        if (!this._EvidenceBookFlags) {
            this.clearBook();
        }
        
        this._EvidenceBookFlags[Number(isSkill)][dataId] = true;
    };

    Game_System.prototype.removeFromBook = function(dataId, isSkill) {
        if (this._EvidenceBookFlags) {
            this._EvidenceBookFlags[Number(isSkill)][dataId] = false;
        }
    };

    Game_System.prototype.completeBook = function() {
        var i, j;
        this.clearBook();
        for (i = 1; i < $dataItems.length; i++) {
            this._EvidenceBookFlags[0][i] = true;
        }
        
        for (j = 1; j < $dataSkills.length; j++) {
            this._EvidenceBookFlags[1][j] = true;
        }
    };

    Game_System.prototype.clearBook = function() {
        this._EvidenceBookFlags = [new Array(0),new Array(0)];
    };

    Game_System.prototype.isInBook = function(item, isSpell) {
        if (this._EvidenceBookFlags && item) {
            return !!this._EvidenceBookFlags[Number(isSpell)][item.id];
        } else {
            return false;
        }
    };

    var _Game_Party_gainItem = Game_Party.prototype.gainItem;
    Game_Party.prototype.gainItem = function(item, amount, includeEquip) {
        _Game_Party_gainItem.call(this, item, amount, includeEquip);
        if (item && amount > 0) {
            if (DataManager.isItem(item)) {
                $gameSystem.addToBook(item.id, false);
            }
        }
    };
    
    var _Game_Actor_learnSkill = Game_Actor.prototype.learnSkill;
    Game_Actor.prototype.learnSkill = function(skillId) {
        _Game_Actor_learnSkill.call(this, skillId);

        $gameSystem.addToBook(skillId, true);
    };

    Game_System.prototype.addItemAlt = function(dataId) {
        if (!this._EvidenceBookFlagsAlt) {
            this.clearItemAlt();
        }

        this._EvidenceBookFlagsAlt.push(dataId);
    };

    Game_System.prototype.clearItemAlt = function() {
        this._EvidenceBookFlagsAlt = [];
    };

    // Scene_EvidenceBook
    //////////////////////////////////////
    
    function Scene_EvidenceBook() {
        this.initialize.apply(this, arguments);
    }

    Scene_EvidenceBook.prototype = Object.create(Scene_MenuBase.prototype);
    Scene_EvidenceBook.prototype.constructor = Scene_EvidenceBook;

    Scene_EvidenceBook.prototype.initialize = function() {
        Scene_MenuBase.prototype.initialize.call(this);
    };

    Scene_EvidenceBook.prototype.create = function() {
        Scene_MenuBase.prototype.create.call(this);
        
        var titleHeight = 28;
        
        //Galv.CI.offset.y += 1;
        
        this._title = new Window_EvidenceBookTitle(0, 0, Graphics.boxWidth, titleHeight);
        this._indexWindow = new Window_EvidenceBookIndex(0, titleHeight);
        
        this._indexWindow.setHandler('cancel', this.popScene.bind(this));
        var wy = titleHeight + this._indexWindow.height;
        var ww = Graphics.boxWidth;
        var wh = Graphics.boxHeight - wy;
        this._statusWindow = new Window_EvidenceBookStatus(0, wy, ww, wh);
        this.addWindow(this._title);
        this.addWindow(this._indexWindow);
        this.addWindow(this._statusWindow);
        this._indexWindow.setStatusWindow(this._statusWindow);
    };
    
    // Window_EvidenceBookTitle
    //////////////////////////////////////
    
    function Window_EvidenceBookTitle() {
        this.initialize.apply(this, arguments);
    }

    Window_EvidenceBookTitle.prototype = Object.create(Window_Base.prototype);
    Window_EvidenceBookTitle.prototype.constructor = Window_EvidenceBookTitle;

    Window_EvidenceBookTitle.prototype.initialize = function(x, y, width, height) {
        Window_Base.prototype.initialize.call(this, x, y, width, height);
        this.refresh();
    };

    Window_EvidenceBookTitle.prototype.refresh = function() {
        var x = 0;
        var y = 0;
        var text = '';

        this.contents.clear();
        
        switch(itemTypeIndex){
            case 1:
                text += 'Keys';
                break;
            case 2:
                text += 'Statements';
                break;
            case 3:
                text += 'Notebook';
                break;
            case 4:
                text += 'Traits';
                break;
            case 5:
                text += 'Facts';
                break;
            case 6:
                text += 'Power';
                break;
            case 7:
                text += 'Heart';
                break;
            case 8:
                text += '???';
                break;
            case 9:
                text += 'Psycho Skills';
                break;
        }
        
        text += '';

        this.drawIcon(130, x + 32, y);
        this.drawIcon(131, (this.contentsWidth() - this.lineHeight()) - 32, y);
        this.drawText(text, x, y, this.contentsWidth(), 'center');
    };
    
    // Window_EvidenceBookIndex
    //////////////////////////////////////

    function Window_EvidenceBookIndex() {
        this.initialize.apply(this, arguments);
    }

    Window_EvidenceBookIndex.prototype = Object.create(Window_Selectable.prototype);
    Window_EvidenceBookIndex.prototype.constructor = Window_EvidenceBookIndex;

    Window_EvidenceBookIndex.lastTopRow = 0;
    Window_EvidenceBookIndex.lastIndex  = 0;

    Window_EvidenceBookIndex.prototype.initialize = function(x, y) {
        var width = Graphics.boxWidth;
        var height = this.fittingHeight(3);
        
        if(itemTypeIndex == 5 || itemTypeIndex == 1 || itemTypeIndex == 6 || itemTypeIndex == 7){
            height = this.fittingHeight(4);
        }
        
        if(itemTypeIndex == 9){
            height = this.fittingHeight(1);
        }
        
        Window_Selectable.prototype.initialize.call(this, x, y, width, height);
        this.refresh();
        this.setTopRow(Window_EvidenceBookIndex.lastTopRow);
        this.select(Window_EvidenceBookIndex.lastIndex);
        this.activate();
    };

    Window_EvidenceBookIndex.prototype.maxCols = function() {
        return 1;
    };

    Window_EvidenceBookIndex.prototype.maxItems = function() {
        return this._list ? this._list.length : 0;
    };

    Window_EvidenceBookIndex.prototype.setStatusWindow = function(statusWindow) {
        this._statusWindow = statusWindow;
        this.updateStatus();
    };

    Window_EvidenceBookIndex.prototype.update = function() {
        Window_Selectable.prototype.update.call(this);
        this.updateStatus();
    };

    Window_EvidenceBookIndex.prototype.updateStatus = function() {
        if (this._statusWindow) {
            var item = this._list[this.index()];
            this._statusWindow.setItem(item);
        }
    };

    Window_EvidenceBookIndex.prototype.refresh = function() {
        var i, item, j, spell, allow, allowB;
        this._list = [];
        for (i = 1; i < $dataItems.length; i++) {
            item = $dataItems[i];
            if (item.name && item.meta.book !== 'no' && $gameSystem.isInBook(item, false)) {
                allow = false;
                
                if(item.itypeId === itemTypeIndex){
                    allow = true;
                }
                
                // if view all clues (Facts & Temptations)
                if(item.itypeId === 4 && itemTypeIndex === 3){
                    allow = true;
                }
                
                // if view only facts
                if(item.itypeId === 3 && itemTypeIndex === 5){
                    allow = true;
                }
                
                if(allow){
                    this._list.push(item);
                }
            }
        }
        
        for (j = 1; j < $dataSkills.length; j++) {
            spell = $dataSkills[j];
            if (spell.name && spell.meta.book !== 'no' && $gameSystem.isInBook(spell, true)) {
                allowB = false;
                
                if(spell.stypeId === (itemTypeIndex - 6)){ // 6 == 0, 7 == 1, etc == 2
                    allowB = true;
                }
                
                if(itemTypeIndex === 9){ // 9 == all spells
                    allowB = true;
                }
                
                if(allowB){
                    this._list.push(spell);
                }
            }
        }
        
        this.createContents();
        this.drawAllItems();
    };
    
    Window_EvidenceBookIndex.prototype.drawAllItems = function() {
        var topIndex = this.topIndex();
        var bookEmpty = true;
        var isSpell = (itemTypeIndex >= 6);
        
        for (var i = 0; i < this.maxPageItems(); i++) {
            var index = topIndex + i;
            if (index < this.maxItems()) {
                this.drawItem(index);
                if ($gameSystem.isInBook(this._list[index], isSpell)) {
                    bookEmpty = false;
                }
            }
        }
        
        // if bookEmpty - Draw None text
        if(bookEmpty){
            var rect = this.itemRect(index);
            var width = rect.width - this.textPadding();
            var iw = Window_Base._iconWidth + 4;
            this.drawText('None', rect.x + iw, 0, width - iw - 6);
        }
    };

    Window_EvidenceBookIndex.prototype.drawItem = function(index) {
        var item = this._list[index];
        var rect = this.itemRect(index);
        var padding = 6;
        var width = rect.width - this.textPadding() - padding;
        var isSpell = (itemTypeIndex >= 6);

        if ($gameSystem.isInBook(item, isSpell)) {
            this.drawItemName(item, rect.x + padding, rect.y - 1, width);
            if(isSpell == false){
                if(item.id == 103 || item.id == 107){
                    this.drawText('x'+$gameParty.numItems(item), rect.x + padding, rect.y - 1, width, 'right');
                }

                if(item.id == gVar(54)){
                    this.drawIcon(90, rect.x + rect.width - 16, rect.y);
                }
            } else {
                if ($gameSystem._EvidenceBookFlagsAlt && $gameSystem._EvidenceBookFlagsAlt.indexOf(item.id) > -1){
                    this.drawIcon(76, rect.x + rect.width - 16, rect.y);
                } else {
                    this.drawIcon(78, rect.x + rect.width - 16, rect.y);
                }
            }
        } else {
            var iw = Window_Base._iconWidth + padding + 2;
            this.drawText(unknownData, rect.x + iw, rect.y, width - iw);
        }
    };
    
    Window_EvidenceBookIndex.prototype.isOkEnabled = function() {
        debugConsole("isOk: " + allowSelect);
        return allowSelect;
    }
    
    Window_EvidenceBookIndex.prototype.processOk = function() {
        var returnIndex = 0;
        var item = this._list[this.index()];
        
        if(item != null){
            returnIndex = Number(item.id);
        
            if(allowSelect){
                str = JSON.stringify(parameters, null, 4); // (Optional) beautiful indented output.
                $gameVariables.setValue(selectIndexVariable,returnIndex);
                $gameSystem.addItemAlt(returnIndex);
                
                this.playOkSound();
                this.updateInputData();
                this.deactivate();
                this.callCancelHandler();
                Window_EvidenceBookIndex.lastTopRow = 0;
                Window_EvidenceBookIndex.lastIndex = 0;
            }
        }
        
        debugConsole(returnIndex);
    };

    Window_EvidenceBookIndex.prototype.processCancel = function() {
        //Galv.CI.offset.y -= 1;
        $gameVariables.setValue(selectIndexVariable,0);
        Window_Selectable.prototype.processCancel.call(this);
        Window_EvidenceBookIndex.lastTopRow = this.topRow();
        Window_EvidenceBookIndex.lastIndex = this.index();
    };
    
    // Window_EvidenceBookStatus
    //////////////////////////////////////

    function Window_EvidenceBookStatus() {
        this.initialize.apply(this, arguments);
    }

    Window_EvidenceBookStatus.prototype = Object.create(Window_Base.prototype);
    Window_EvidenceBookStatus.prototype.constructor = Window_EvidenceBookStatus;

    Window_EvidenceBookStatus.prototype.initialize = function(x, y, width, height) {
        Window_Base.prototype.initialize.call(this, x, y, width, height);
    };

    Window_EvidenceBookStatus.prototype.setItem = function(item) {
        if (this._item !== item) {
            this._item = item;
            this.refresh();
        }
    };

    Window_EvidenceBookStatus.prototype.refresh = function() {
        var item = this._item;
        var x = 0;
        var y = 0;
        
        var isSpell = (itemTypeIndex >= 6);

        this.contents.clear();

        if (!item || !$gameSystem.isInBook(item, isSpell)) {
            return;
        }
        
        this.drawTextEx(item.description, x, y);
    };

})();
